
{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRnnLogTx.PAS, released on 2000-12-01.

The Initial Developer of the Original Code is Michael Rynn.
Portions created by Michael Rynn are Copyright (C) 2000  Michael Rynn.
All Rights Reserved.

Contributor(s): ______________________________________.

Last Modified: 2000-12-01

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}

{$I JCL.INC}

unit JvRnnLogTx;

interface
Uses WinProcs,WinTypes,Classes;
{$IFDEF WIN32}
  {$DEFINE THREAD_ACCESS}
{$ENDIF}
type
  TNotifyAppend = procedure(Sender:TObject; const s : string) of object;
  PTextfile = ^TextFile;
  TRnnLogTextFile = class(TObject)
  private
    FLog : Textfile;
    FName :  string;
    FIsOpen : boolean;
    FBuffer : Pointer;
    FUnflushed : integer;
    FNotifyAppend : TNotifyAppend;
{$IFDEF THREAD_ACCESS}
    FLock: TRTLCriticalSection; {experimental}
{$ENDIF}
  protected
    function GetLogfile : PTextfile;
    procedure LockAccess;
    procedure UnlockAccess;
  public
    constructor Create(const path : string);
    destructor  Destroy; override;
    procedure OpenAppend;
    procedure OpenRewrite;
    procedure Close;
    procedure Flush;
    procedure AppendStr(slist : TStrings);
    procedure Append(const s : string);
    procedure SetBuffer;
    {$IFDEF WIN32}
    function  GetSize : LongInt;
    {$ENDIF}
    property Open : boolean read FIsOpen;
    property LogFile : PTextFile read GetLogFile;
    property FileName : string read FName;
    property Unflushed : integer read FUnflushed;
    property OnAppendStr : TNotifyAppend
      read FNotifyAppend write FNotifyAppend;
  end;

function GetFileLength(const fname : string) : integer;

function LogsAreOpen : boolean;

procedure CloseAllLogFiles;

implementation
Uses Dialogs,SysUtils;

const
	kTextFileBufSize = 4096;
{$IFDEF THREAD_ACCESS}
var
	sLogList : TThreadList;
{$ELSE}
var
	sLogList : TList;
{$ENDIF}

function GetFileLength(const fname : string) : integer;
var
  s : TSearchRec;
begin
  if (FindFirst(fname, faAnyFile, s)=0) then
  begin
    Result := s.Size;
    FindClose(s);
  end
  else
    Result := -1;
end;

procedure TRnnLogTextFile.SetBuffer;
begin
  if FBuffer = nil then
  begin
    GetMem(FBuffer,kTextFileBufSize);
    SetTextBuf(FLog,FBuffer^, kTextFileBufSize);
  end;
end;

function TRnnLogTextFile.GetLogfile : PTextfile;
begin
  Result := @FLog;
end;

constructor TRnnLogTextFile.Create(const path : string);
begin
  inherited Create;
  {$IFDEF THREAD_ACCESS}
  InitializeCriticalSection(FLock);
  {$ENDIF}
  FName := path;
  FBuffer := nil;
  AssignFile(FLog,FName);
  if sLogList=nil then
  begin
  {$IFDEF THREAD_ACCESS}
    sLogList := TThreadList.Create;
  {$ELSE}
    sLogList := TList.Create;
  {$ENDIF}
    AddExitProc(CloseAllLogFiles);
  end;
  sLogList.Add(SELF);
end;


procedure TRnnLogTextFile.OpenAppend;
begin
  LockAccess;
  try
    Close;
    if not FileExists(FName) then
    begin
      OpenRewrite;
      exit;
    end;
    System.Append(FLog);
    SetBuffer;
    FIsOpen := TRUE;
  finally
    UnlockAccess;
  end;
end;

procedure TRnnLogTextFile.UnlockAccess;
begin
{$IFDEF THREADS}
  LeaveCriticalSection(FLock);
{$ENDIF}
end;

procedure TRnnLogTextFile.LockAccess;
begin
{$IFDEF THREADS}
  EnterCriticalSection(FLock);
{$ENDIF}
end;

procedure TRnnLogTextFile.Flush;
begin
  if FIsOpen then begin
    System.Flush(FLog);
    FUnflushed := 0;
  end;
end;

procedure TRnnLogTextFile.OpenRewrite;
begin
  LockAccess;
  try
    Close;
    System.Rewrite(FLog);
    SetBuffer;
    FIsOpen := TRUE;
  finally
    UnlockAccess;
  end;
end;

procedure TRnnLogTextFile.Close; 
begin
  LockAccess;
  try
    if FIsOpen then
    begin
      Flush;
      CloseFile(FLog);
      FIsOpen := FALSE;
    end;
  finally
    UnlockAccess;
  end;
end;

{$IFDEF WIN32}
function TRnnLogTextFile.GetSize : LongInt;
begin
	Result := system.FileSize(FLog);
end;
{$ENDIF}

destructor TRnnLogTextFile.Destroy;
begin
  try
    LockAccess;
    try
      sLogList.Remove(SELF);
      Flush;
      if FBuffer <> nil then
      begin
        FreeMem(FBuffer,kTextFileBufSize);
        FBuffer := nil;
      end;
      Close;
    finally
       UnlockAccess;
       {$IFDEF THREAD_ACCESS}
       DeleteCriticalSection(FLock);
       {$ENDIF}
    end;
  except
    on e:exception do
      ShowMessage(e.message);
  end;
  inherited Destroy;
end;

procedure TRnnLogTextFile.AppendStr(slist : TStrings);
var
  i : integer;
begin
  LockAccess;
  try
    for i := 0 to slist.count-1 do
    begin
      Append(slist[i]);
    end;
  finally
    UnlockAccess;
  end;
end;

procedure TRnnLogTextFile.Append(const s : string);
begin
  LockAccess;
  try
    if not FIsOpen then
      if FileExists(FName) then
        OpenAppend
      else
        OpenRewrite;
    writeln(FLog,s);
    Inc(FUnflushed);
  finally
    UnlockAccess;
  end;
  if Assigned(FNotifyAppend) then
    FNotifyAppend(SELF,s);
end;

function LogsAreOpen : boolean;
var
  slist : TList;
begin
{$IFDEF THREAD_ACCESS}
  slist := sLogList.LockList;
{$ELSE}
  slist := sLogList;
{$ENDIF}
  Result := (slist<>nil) and (slist.Count > 0);
{$IFDEF THREAD_ACCESS}
  sLogList.UnlockList;
{$ENDIF}
end;

procedure CloseAllLogFiles;
var
  logfile:TRnnLogTextFile;
  slist : TList;
begin
  if sLogList=nil
    then exit;
  {$IFDEF THREAD_ACCESS}
  slist := sLogList.LockList;
  {$ELSE}
  slist := sLogList;
  {$ENDIF}
  try
    while slist.Count > 0 do
    begin
      logfile := TRnnLogTextFile(slist.Last);
      slist.Delete(slist.count-1);
      logfile.free;
    end;
  finally
    {$IFDEF THREAD_ACCESS}
    sLogList.UnlockList;
    {$ENDIF}
    sLogList.Free;
    sLogList := nil;
  end;
end;

end.
